//
//  ViewController.m
//  BlogPost_NSCoding
//
//  Created by Darrell Kress on 10/14/13.
//  Copyright (c) 2013 Darrell Kress. All rights reserved.
//

#import "ViewController.h"


@interface Publisher : NSObject<NSCoding>
@property NSString* name;
@end

@implementation Publisher

-(id)initWithName:(NSString*)name
{
    self = [super init];
    if (self)
    {
        self.name = name;
    }
    return self;
}

-(void)encodeWithCoder:(NSCoder *)aCoder
{
    [aCoder encodeObject:self.name forKey:@"name"];
}
-(id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super init];
     self.name = (NSString*)[aDecoder decodeObjectForKey:@"name"];
     return self;
}
@end

@interface Book : NSObject<NSCoding>
@property int pageCount;
@property NSString* title;
@property Publisher* publisher;
@end
@implementation Book

-(void)encodeWithCoder:(NSCoder *)aCoder
{
    [aCoder encodeObject:self.title forKey:@"title"];
    [aCoder encodeInt:self.pageCount forKey:@"pageCount"];
    [aCoder encodeObject:self.publisher forKey:@"publisher"];
}
-(id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super init];
    self.title = (NSString*)[aDecoder decodeObjectForKey:@"title"];
    self.pageCount = [aDecoder decodeIntForKey:@"pageCount"];
    self.publisher = (Publisher*)[aDecoder decodeObjectForKey:@"publisher"];
    return self;
}

@end


@interface ViewController ()
{
    UIButton* _saveData;
    UIButton* _loadData;
    
    UIButton* _saveDataToFile;
    UIButton* _loadDataFromFile;
    
    UILabel* _label1;
    UILabel* _label2;
}
@end

@implementation ViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.

    _saveData = [[UIButton alloc]initWithFrame:CGRectMake(0, 40, 100, 100)];
    _saveData.backgroundColor = [UIColor yellowColor];
    UILabel* label = [[UILabel alloc]initWithFrame:CGRectMake(0, 0, 100, 100)];
    label.text = @"Save Data - Memory";
    label.numberOfLines = 0;
    [_saveData addSubview:label];
    [self.view addSubview:_saveData];
    UITapGestureRecognizer* tap1 = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(tapToSave)];
    [_saveData addGestureRecognizer:tap1];
    
    _loadData = [[UIButton alloc]initWithFrame:CGRectMake(100, 40, 100, 100)];
    _loadData.backgroundColor = [UIColor greenColor];
    label = [[UILabel alloc]initWithFrame:CGRectMake(0, 0, 100, 100)];
    label.numberOfLines = 0;
    label.text = @"Load Data - Memory";
    [_loadData addSubview:label];
    [self.view addSubview:_loadData];
    UITapGestureRecognizer* tap2 = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(tapToLoad)];
    [_loadData addGestureRecognizer:tap2];
    
    _saveDataToFile = [[UIButton alloc]initWithFrame:CGRectMake(0, 150, 100, 100)];
    _saveDataToFile.backgroundColor = [UIColor yellowColor];
    label = [[UILabel alloc]initWithFrame:CGRectMake(0, 0, 100, 100)];
    label.text = @"Save Data - File";
    label.numberOfLines = 0;
    [_saveDataToFile addSubview:label];
    [self.view addSubview:_saveDataToFile];
    UITapGestureRecognizer* tap3 = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(tapToSave2)];
    [_saveDataToFile addGestureRecognizer:tap3];
    

    _loadDataFromFile = [[UIButton alloc]initWithFrame:CGRectMake(100, 150, 100, 100)];
    _loadDataFromFile.backgroundColor = [UIColor greenColor];
    label = [[UILabel alloc]initWithFrame:CGRectMake(0, 0, 100, 100)];
    label.numberOfLines = 0;
    label.text = @"Load Data - File";
    [_loadDataFromFile addSubview:label];
    [self.view addSubview:_loadDataFromFile];
    UITapGestureRecognizer* tap4 = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(tapToLoad2)];
    [_loadDataFromFile addGestureRecognizer:tap4];

    _label1 = [[UILabel alloc]initWithFrame:CGRectMake(200,40,100,100)];
    [self.view addSubview:_label1];
    
    _label2 = [[UILabel alloc]initWithFrame:CGRectMake(200,150,100,100)];
    [self.view addSubview:_label2];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

NSData* _savedData;
-(NSMutableArray*)makeData
{
    NSMutableArray* data = [[NSMutableArray alloc]init];
    
    Book* b1 = [[Book alloc]init];
    b1.title = @"Book 1";
    b1.pageCount = 10;
    b1.publisher = [[Publisher alloc]initWithName:@"First Publisher"];
    
    Book* b2 = [[Book alloc]init];
    b2.title = @"Book 2";
    b2. pageCount = 20;
    b2.publisher = [[Publisher alloc]initWithName:@"Second Publisher"];
    
    Book* b3 = [[Book alloc]init];

    [data addObject:b1];
    [data addObject:b2];
    [data addObject:b3];
    
    return data;
}

-(void)tapToSave
{
    _savedData = [NSKeyedArchiver archivedDataWithRootObject:[self makeData]];
    _label1.text = @"saved";
}

-(void)tapToLoad
{
    if (_savedData != nil)
    {
        NSMutableArray* bookData = (NSMutableArray*) [NSKeyedUnarchiver unarchiveObjectWithData:_savedData];
        _label1.text = [NSString stringWithFormat:@"%d" ,(int)bookData.count ];
    
    }
    _savedData = nil;
}

-(void)tapToSave2
{
    [self save:@"MyFile" withData:[self makeData]];
    _label2.text = @"Saved";
}

-(void)ensureDiretory:(NSString*)path
{
    NSFileManager* manager = [NSFileManager defaultManager];
    if(![manager fileExistsAtPath:path])
    {
        [manager createDirectoryAtPath:path withIntermediateDirectories:YES attributes:nil error:nil];
    }
}

-(void)save:(NSString*)key withData:(NSObject*)data
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
    NSString* rootPath = paths[0];
    rootPath = [rootPath stringByAppendingPathComponent:@"DataStorage"];
    [self ensureDiretory:rootPath];
    
    rootPath = [rootPath stringByAppendingPathComponent:key];
    rootPath = [rootPath stringByAppendingPathExtension:@"data"];
    
    [NSKeyedArchiver archiveRootObject:data toFile:rootPath];
}

-(NSObject*)load:(NSString*)key
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
    NSString* rootPath = paths[0];
    rootPath = [rootPath stringByAppendingPathComponent:@"DataStorage"];
    [self ensureDiretory:rootPath];
    
    rootPath = [rootPath stringByAppendingPathComponent:key];
    rootPath = [rootPath stringByAppendingPathExtension:@"data"];
    
    return (NSMutableArray*)[NSKeyedUnarchiver unarchiveObjectWithFile:rootPath];
}

-(void)tapToLoad2
{
    // loading from a file
    NSMutableArray* data = (NSMutableArray*)[self load:@"MyFile"];
    _label2.text = [NSString stringWithFormat:@"%d" ,(int)data.count ];
}
@end

